/*
 * Copyright © 2012 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Kevin DuBois <kevin.dubois@canonical.com>
 */

#include "mir/compositor/back_buffer_strategy.h"

#include "buffer_bundle.h"
#include "temporary_buffers.h"

namespace mc=mir::compositor;
namespace geom=mir::geometry;

mc::TemporaryBuffer::TemporaryBuffer(std::shared_ptr<Buffer> const& real_buffer)
    : buffer(real_buffer)
{
}

mc::TemporaryClientBuffer::TemporaryClientBuffer(std::shared_ptr<BufferBundle> const& buffer_swapper)
    : TemporaryBuffer(buffer_swapper->client_acquire()),
      allocating_swapper(buffer_swapper)
{
}

mc::TemporaryClientBuffer::~TemporaryClientBuffer()
{
    if (auto swapper = allocating_swapper.lock())
        swapper->client_release(buffer);
}

mc::TemporaryCompositorBuffer::TemporaryCompositorBuffer(
    std::shared_ptr<BackBufferStrategy> const& back_buffer_strategy)
    : TemporaryBuffer(back_buffer_strategy->acquire()),
      back_buffer_strategy(back_buffer_strategy)
{
}

mc::TemporaryCompositorBuffer::~TemporaryCompositorBuffer()
{
    if (auto strategy = back_buffer_strategy.lock())
        strategy->release(buffer);
}

geom::Size mc::TemporaryBuffer::size() const
{
    return buffer->size();
}

geom::Stride mc::TemporaryBuffer::stride() const
{
    return buffer->stride();
}

geom::PixelFormat mc::TemporaryBuffer::pixel_format() const
{
    return buffer->pixel_format();
}

mc::BufferID mc::TemporaryBuffer::id() const
{
    return buffer->id();
}

void mc::TemporaryBuffer::bind_to_texture()
{
    buffer->bind_to_texture();
}

std::shared_ptr<MirNativeBuffer> mc::TemporaryBuffer::native_buffer_handle() const
{
    return buffer->native_buffer_handle();
}
